--==============================================================================
-- CERN (BE-CO-HT)
-- Pulse trigger for pulse converter boards
--==============================================================================
--
-- author: Theodor Stana (t.stana@cern.ch)
--
-- date of creation: 2014-01-28
--
-- version: 1.0
--
-- description:
--
-- dependencies:
--      genram_pkg : git://ohwr.org/hdl-core-lib/general-cores.git
--
-- references:
--
--==============================================================================
-- GNU LESSER GENERAL PUBLIC LICENSE
--==============================================================================
-- This source file is free software; you can redistribute it and/or modify it
-- under the terms of the GNU Lesser General Public License as published by the
-- Free Software Foundation; either version 2.1 of the License, or (at your
-- option) any later version. This source is distributed in the hope that it
-- will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-- See the GNU Lesser General Public License for more details. You should have
-- received a copy of the GNU Lesser General Public License along with this
-- source; if not, download it from http://www.gnu.org/licenses/lgpl-2.1.html
--==============================================================================
-- last changes:
--    2014-01-28   Theodor Stana     File created
--==============================================================================
-- TODO: -
--==============================================================================

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.genram_pkg.all;

entity conv_man_trig is
  generic
  (
    -- Number of conversion channels
    g_nr_chan : positive := 6
  );
  port
  (
    -- Clock, active-low inputs
    clk_i     : in  std_logic;
    rst_n_i   : in  std_logic;

    -- Control inputs from conv_regs
    reg_ld_i : in  std_logic;
    reg_i    : in  std_logic_vector(7 downto 0);

    -- One-clock pulse output
    p_o       : out std_logic_vector(g_nr_chan downto 1)
  );
end entity conv_man_trig;


architecture behav of conv_man_trig is

  --============================================================================
  -- Type declarations
  --============================================================================
  -- Type for the "password" array
  type t_pass_arr is array(integer range <>) of std_logic_vector(7 downto 0);

  -- FSM type
  type t_state is
    (
      IDLE,
      PASS1,
      PASS2,
      PASS3,
      GEN
    );

  --============================================================================
  -- Constant declarations
  --============================================================================
  constant c_pass_arr : t_pass_arr(0 to 3) := (x"de", x"ad", x"be", x"ef");

  --============================================================================
  -- Function and procedures declaration
  --============================================================================
  procedure f_change_state (
    signal   ld     : in  std_logic;
    signal   pass   : in  std_logic_vector(7 downto 0);
    constant idx    : in  integer;
    signal   state  : out t_state;
    constant nstate : in  t_state
  ) is
  begin
    if (ld = '1') then
      if (pass = c_pass_arr(idx)) then
        state <= nstate;
      else
        state <= IDLE;
      end if;
    end if;
  end procedure f_change_state;

  --============================================================================
  -- Signal declarations
  --============================================================================
  -- Signal for the current state of the FSM
  signal state : t_state;

  -- "Password" and channel number signals
  signal pass  : std_logic_vector(7 downto 0);
  signal chnr  : std_logic_vector(f_log2_size(g_nr_chan)-1 downto 0);

--==============================================================================
--  architecture begin
--==============================================================================
begin

  --============================================================================
  -- FSM logic
  --============================================================================
  -- First, assign the password, channel enable and channel number signals
  pass <= reg_i;
  chnr <= reg_i(f_log2_size(g_nr_chan)-1 downto 0);

  -- Then, the process for the FSM
  p_fsm : process (clk_i)
  begin
    if rising_edge(clk_i) then
      if (rst_n_i = '0') then
        state <= IDLE;
        p_o   <= (others => '0');
      else
        case state is
          when IDLE =>
            p_o <= (others => '0');
            f_change_state(reg_ld_i, pass, 0, state, PASS1);
          when PASS1 =>
            f_change_state(reg_ld_i, pass, 1, state, PASS2);
          when PASS2 =>
            f_change_state(reg_ld_i, pass, 2, state, PASS3);
          when PASS3 =>
            f_change_state(reg_ld_i, pass, 3, state, GEN);
          when GEN =>
            if (reg_ld_i = '1') then
              p_o(to_integer(unsigned(chnr))) <= '1';
              state <= IDLE;
            end if;
          when others =>
            state <= IDLE;
        end case;
      end if;
    end if;
  end process p_fsm;

end architecture behav;
--==============================================================================
--  architecture end
--==============================================================================
